# -*- encoding: utf-8 -*-
"""
License: MIT
Copyright (c) 2019 - present AppSeed.us
"""
from flask import Flask, url_for, session, render_template
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.progress import Progress, TextColumn
from rich.columns import Columns
from rich.traceback import install, Trace, Traceback
from rich.rule import Rule
from rich.logging import RichHandler
from waitress import serve
from importlib import import_module
from os import path
from cryptography import fernet
from io import StringIO
import base64
import threading
import time
import websocket
import traceback
import logging
import rich
import json
import sys
import os

log = logging.getLogger('werkzeug')
dashlog = logging.getLogger('reddash')

console = Console()
oldexcepthook = install()
progress = Progress("{task.description}", TextColumn("{task.fields[status]}\n"))

logging.basicConfig(format="%(message)s", handlers=[RichHandler(console=progress)])

class Lock:
    def __init__(self):
        self.lock = threading.Lock()

    def __enter__(self):
        self.lock.acquire()

    def __exit__(self, *args):
        self.lock.release()

global __version__
__version__ = "0.1.3a"
__author__ = "Neuro Assassin#4779"

# In case the dashboard cog isn't loaded
global defaults
defaults = {
    'botname': 'Red Discord Bot',
    'botavatar': 'https://cdn.discordapp.com/icons/133049272517001216/a_aab012f3206eb514cac0432182e9e9ec.gif?size=1024',
    'botinfo': 'Hello, welcome to the Red Discord Bot dashboard!  Here you can see basic information, commands list and even interact with your bot!  Unfortunately, this dashboard is not connected to any bot currently, so none of these features are available.  If you are the owner of the bot, please load the dashboard cog from Toxic Cogs.',
    'owner': 'Cog Creators'
}

global url
url = "ws://localhost:"

global app
app = None

global lock
lock = Lock()

global running
running = True

def update_variables(method, task):
    progress.update(task, status="[bold green]Running[/bold green]")
    progress.refresh()
    try:
        while True:
            global app
            # Different wait times based on method, commands should be called less due to how much data it is
            if method == "DASHBOARDRPC__GET_VARIABLES":
                _id = 1
                time.sleep(app.interval)
            else:
                _id = 2
                time.sleep(app.interval * 2)

            global running
            if not running:
                progress.update(task, status="[bold red]Killed[/bold red]")
                progress.refresh()
                return

            request = {
                "jsonrpc": "2.0",
                "id": _id,
                "method": method,
                "params": []
            }
            with app.lock:
                # This needs to be inside the lock, or both threads will create a websocket
                if not app.ws:
                    app.ws = websocket.WebSocket()
                    try:
                        app.ws.connect(url)
                    except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                        app.ws.close()
                        app.ws = None
                        continue
                try:
                    app.ws.send(json.dumps(request))
                except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                    dashlog.warning("Connection reset")
                    app.ws.close()
                    app.ws = None
                    continue
                    
                try:
                    result = json.loads(app.ws.recv())
                except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                    dashlog.warning("Connection reset")
                    app.ws.close()
                    app.ws = None
                    continue
                if 'error' in result:
                    if result['error']['message'] == "Method not found":
                        if method == "DASHBOARDRPC__GET_VARIABLES":
                            app.variables = {}
                        app.ws.close()
                        app.ws = None
                        continue
                    dashlog.error(result['error'])
                    app.ws.close()
                    app.ws = None
                    continue
                if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                    # Dashboard cog unloaded, disconnect
                    if method == "DASHBOARDRPC__GET_VARIABLES":
                        app.variables = {}
                    app.ws.close()
                    app.ws = None
                    continue
            if method == "DASHBOARDRPC__GET_VARIABLES":
                app.variables = result['result']
            else:
                app.commanddata = result['result']
            app.variables["disconnected"] = False
    except Exception as e:
        progress.update(task, status="[bold red]Stopped[/bold red]")
        progress.refresh()
        with console:
            if console.is_terminal:
                console.print(progress._live_render.position_cursor())
            console.print_exception()
            if console.is_terminal:
                console.print(progress._live_render)

def update_version(task):
    progress.update(task, status="[bold green]Running[/bold green]")
    progress.refresh()
    try:
        while True:
            time.sleep(1)
            global running
            if not running:
                progress.update(task, status="[bold red]Killed[/bold red]")
                progress.refresh()
                return
            if app.ws and app.ws.connected:
                request = {
                    "jsonrpc": "2.0",
                    "id": 0,
                    "method": "DASHBOARDRPC__CHECK_VERSION",
                    "params": [app.rpcversion]
                }
                with app.lock:
                    if not app.ws:
                        app.ws = websocket.WebSocket()
                        try:
                            app.ws.connect(url)
                        except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                            app.ws.close()
                            app.ws = None
                            continue

                    try:
                        app.ws.send(json.dumps(request))
                    except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                        continue
                        
                    try:
                        result = json.loads(app.ws.recv())
                    except (ConnectionRefusedError, websocket._exceptions.WebSocketConnectionClosedException, ConnectionResetError):
                        continue

                    if 'error' in result:
                        continue

                    if "disconnected" in result['result']:
                        continue

                    if result['result']['v'] != app.rpcversion and app.rpcversion != 0:
                        dashlog.warning("RPC websocket behind.  Closing and restarting...")
                        app.ws.close()
                        app.ws = websocket.WebSocket()
                        app.ws.connect(url)
                    app.rpcversion = result['result']['v']
    except Exception as e:
        progress.update(task, status="[bold red]Stopped[/bold red]")
        progress.refresh()
        with console:
            if console.is_terminal:
                console.print(progress._live_render.position_cursor())
            console.print_exception()
            if console.is_terminal:
                console.print(progress._live_render)

def check_if_connected(task):
    while True:
        global running
        if not running:
            app.ws.close()
            del app.ws
            progress.update(task, status="[bold red]Websocket Killed[/bold red]")
            progress.refresh()
            return
        time.sleep(0.1)
        if not (app.ws and app.ws.connected):
            progress.update(task, status="[bold red]Disconnected[/bold red]")
            progress.refresh()
        else:
            progress.update(task, status="[bold green]Connected[/bold green]")
            progress.refresh()

def register_blueprints(app):
    for module_name in ('base', 'home'):
        module = import_module('reddash.app.{}.routes'.format(module_name))
        app.register_blueprint(module.blueprint)

    @app.errorhandler(404)
    def page_not_found(e):
        return render_template('page-404.html'), 404

def apply_themes(app):
    """
    Add support for themes.

    If DEFAULT_THEME is set then all calls to
      url_for('static', filename='')
      will modfify the url to include the theme name

    The theme parameter can be set directly in url_for as well:
      ex. url_for('static', filename='', theme='')

    If the file cannot be found in the /static/<theme>/ location then
      the url will not be modified and the file is expected to be
      in the default /static/ location
    """
    @app.context_processor
    def override_url_for():
        return dict(url_for=_generate_url_for_theme)

    def _generate_url_for_theme(endpoint, **values):
        if endpoint.endswith('static'):
            themename = values.get('theme', None) or \
                app.config.get('DEFAULT_THEME', None)
            if themename:
                theme_file = "{}/{}".format(themename, values.get('filename', ''))
                if path.isfile(path.join(app.static_folder, theme_file)):
                    values['filename'] = theme_file
        return url_for(endpoint, **values)

def add_constants(app):
    @app.context_processor
    def inject_variables():
        global __version__
        if not app.variables:
            return dict(version=__version__, **defaults)
        return dict(version=__version__, **app.variables)

def create_app(host, port, rpcport, interval, debug, dev, selenium=False):
    global url
    global app
    global lock

    url += str(rpcport)
    
    fernet_key = fernet.Fernet.generate_key()
    secret_key = base64.urlsafe_b64decode(fernet_key)

    app = Flask('reddash', static_folder='app/base/static')
    app.ws = None
    app.lock = lock
    app.variables = {}
    app.commanddata = {}
    app.config.from_object(__name__)
    app.config['TEMPLATES_AUTO_RELOAD'] = True
    app.secret_key = secret_key
    app.rpcport = str(rpcport)
    app.rpcversion = 0
    app.interval = interval

    # Cooldowns (Yay!)
    app.cooldowns = {
        "serverprefix": {},
        "adminroles": {},
        "modroles": {}
    }
    
    if selenium:
        app.config['LOGIN_DISABLED'] = True
    register_blueprints(app)
    apply_themes(app)
    add_constants(app)

    if debug:
        log.setLevel(logging.DEBUG)
        dashlog.setLevel(logging.DEBUG)

    table = Table(title="Settings")
    table.add_column("Setting", style="red", no_wrap=True)
    table.add_column("Value", style="blue", no_wrap=True)

    table.add_row("Version", __version__)
    table.add_row("Host", host)
    table.add_row("Webserver port", str(port))
    table.add_row("RPC Port", str(rpcport))
    table.add_row("Update interval", str(interval))
    table.add_row("Environment", "Development" if dev else "Production")
    table.add_row("Logging level", "Debug" if debug else "Warning")

    progress.print(Rule("Red Discord Bot Dashboard - Webserver"))
    disclaimer = "This is an instance of Red Discord Bot Dashboard,\ncreated by Neuro Assassin. This package is\nprotected under an MIT License. Any action\nthat will breach this license (including but not\nlimited to, removal of credits) may result in a\nDMCA request, or possibly more.\n\n\n\nYou can view the license at\nhttps://github.com/NeuroAssassin/\nRed-Dashboard/blob/master/LICENSE."

    vartask = progress.add_task("Update variable task:", status="[bold blue]Starting[/bold blue]")
    cmdtask = progress.add_task("Update command task:", status="[bold blue]Starting[/bold blue]")
    vertask = progress.add_task("Update version task:", status="[bold blue]Starting[/bold blue]")
    contask = progress.add_task("RPC Connected", status="[bold blue]Starting[/bold blue]")

    progress.print(Columns([Panel(table), Panel(disclaimer)], equal=True))

    threads = []
    threads.append(threading.Thread(target=update_variables, args=["DASHBOARDRPC__GET_VARIABLES", vartask], daemon=True))
    threads.append(threading.Thread(target=update_variables, args=["DASHBOARDRPC__GET_COMMANDS", cmdtask], daemon=True))
    threads.append(threading.Thread(target=update_version, args=[vertask], daemon=True))
    threads.append(threading.Thread(target=check_if_connected, args=[contask], daemon=True))

    for t in threads:
        t.start()

    if dev:
        app.run(host=host, port=port, debug=debug)
    else:
        serve(app, host=host, port=port, _quiet=True)
    
    dashlog.fatal("Shutting down...")
    global running
    running = False
    for t in threads:
        t.join()
    
    dashlog.fatal("Webserver terminated")


# -*- encoding: utf-8 -*-
"""
License: MIT
Copyright (c) 2019 - present AppSeed.us
"""

from flask import Blueprint

blueprint = Blueprint(
    'home_blueprint',
    __name__,
    url_prefix='',
    template_folder='templates',
    static_folder='static'
)



# -*- encoding: utf-8 -*-
"""
License: MIT
Copyright (c) 2019 - present AppSeed.us
"""
from reddash.app import app, update_variables
from reddash.app.home import blueprint
from flask import render_template, redirect, url_for, session, request, jsonify, Response
from jinja2 import TemplateNotFound
import websocket
import json
import time
import random
import logging
import datetime

dashlog = logging.getLogger("reddash")

def update_core():
    while True:
        time.sleep(0.5)
        try:
            yield f"data: {app.variables['servers']}, {app.variables['users']}, {app.variables['onlineusers']}\n\n"
        except KeyError:
            yield ""

# --------------------------------------- API ---------------------------------------

@blueprint.route('/api/stream')
def stream():
    return Response(update_core(), mimetype="text/event-stream")

@blueprint.route('/api/getservers')
def getservers():
    if not session.get("id"):
        return jsonify({"status": 0, "message": "Not logged in"})
    try:
        requeststr = {
            "jsonrpc": "2.0",
            "id": 0,
            "method": "DASHBOARDRPC__GET_USERS_SERVERS",
            "params": [int(session['id'])]
        }
        with app.lock:
            app.ws.send(json.dumps(requeststr))
            result = json.loads(app.ws.recv())
            if 'error' in result:
                if result['error']['message'] == "Method not found":
                    return jsonify({"status": 0, "message": "Not connected to bot"})
                dashlog.error(result['error'])
                return jsonify({"status": 0, "message": "Something went wrong"})
            if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                return jsonify({"status": 0, "message": "Not connected to bot"})
            return jsonify({"status": 1, "data": result['result']})
    except:
        return jsonify({"status": 0, "message": "Not connected to bot"})

@blueprint.route('/api/serverprefix', methods=['POST'])
def serverprefix():
    if not session.get("id"):
        return jsonify({"status": 0, "message": "Not logged in"})

    if (end := app.cooldowns["serverprefix"].get(session.get("id"), datetime.datetime.now() - datetime.timedelta(seconds=5))) > datetime.datetime.now():
        return jsonify({"status": 0, "message": f"You are doing that too much.  Try again in {(end - datetime.datetime.now()).seconds} seconds"})
    app.cooldowns["serverprefix"][session.get("id")] = datetime.datetime.now() + datetime.timedelta(seconds=5)

    data = request.json
    guildid = data.get("guild")
    userid = session.get("id")
    method = "set"
    prefixes = data.get("prefixes")
    if not (guildid and prefixes):
        return jsonify({"status": 0, "message": "Guild ID and prefixes must be specified"})
    try:
        requeststr = {
            "jsonrpc": "2.0",
            "id": 0,
            "method": "DASHBOARDRPC_BOTSETTINGS__SERVERPREFIX",
            "params": [int(guildid), int(userid), method, prefixes]
        }
        with app.lock:
            app.ws.send(json.dumps(requeststr))
            result = json.loads(app.ws.recv())
            if 'error' in result:
                if result['error']['message'] == "Method not found":
                    return jsonify({"status": 0, "message": "Not connected to bot"})
                dashlog.error(result['error'])
                return jsonify({"status": 0, "message": "Something went wrong"})
            if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                return jsonify({"status": 0, "message": "Not connected to bot"})
            return jsonify({"status": 1, "data": result['result']})
    except:
        return jsonify({"status": 0, "message": "Not connected to bot"})

@blueprint.route('/api/adminroles', methods=['POST'])
def adminroles():
    if not session.get("id"):
        return jsonify({"status": 0, "message": "Not logged in"})

    if (end := app.cooldowns["adminroles"].get(session.get("id"), datetime.datetime.now() - datetime.timedelta(seconds=5))) > datetime.datetime.now():
        return jsonify({"status": 0, "message": f"You are doing that too much.  Try again in {(end - datetime.datetime.now()).seconds} seconds"})
    app.cooldowns["adminroles"][session.get("id")] = datetime.datetime.now() + datetime.timedelta(seconds=5)

    data = request.json
    guildid = data.get("guild")
    userid = session.get("id")
    method = "set"
    roles = data.get("roles")
    if not guildid:
        return jsonify({"status": 0, "message": "Guild ID must be specified"})
    try:
        requeststr = {
            "jsonrpc": "2.0",
            "id": 0,
            "method": "DASHBOARDRPC_BOTSETTINGS__ADMINROLES",
            "params": [int(guildid), int(userid), method, roles]
        }
        with app.lock:
            app.ws.send(json.dumps(requeststr))
            result = json.loads(app.ws.recv())
            if 'error' in result:
                if result['error']['message'] == "Method not found":
                    return jsonify({"status": 0, "message": "Not connected to bot"})
                dashlog.error(result['error'])
                return jsonify({"status": 0, "message": "Something went wrong"})
            if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                return jsonify({"status": 0, "message": "Not connected to bot"})
            return jsonify({"status": 1, "data": result['result']})
    except:
        return jsonify({"status": 0, "message": "Not connected to bot"})

@blueprint.route('/api/modroles', methods=['POST'])
def modroles():
    if not session.get("id"):
        return jsonify({"status": 0, "message": "Not logged in"})

    if (end := app.cooldowns["modroles"].get(session.get("id"), datetime.datetime.now() - datetime.timedelta(seconds=5))) > datetime.datetime.now():
        return jsonify({"status": 0, "message": f"You are doing that too much.  Try again in {(end - datetime.datetime.now()).seconds} seconds"})
    app.cooldowns["modroles"][session.get("id")] = datetime.datetime.now() + datetime.timedelta(seconds=5)

    data = request.json
    guildid = data.get("guild")
    userid = session.get("id")
    method = "set"
    roles = data.get("roles")
    if not guildid:
        return jsonify({"status": 0, "message": "Guild ID must be specified"})
    try:
        requeststr = {
            "jsonrpc": "2.0",
            "id": 0,
            "method": "DASHBOARDRPC_BOTSETTINGS__MODROLES",
            "params": [int(guildid), int(userid), method, roles]
        }
        with app.lock:
            app.ws.send(json.dumps(requeststr))
            result = json.loads(app.ws.recv())
            if 'error' in result:
                if result['error']['message'] == "Method not found":
                    return jsonify({"status": 0, "message": "Not connected to bot"})
                dashlog.error(result['error'])
                return jsonify({"status": 0, "message": "Something went wrong"})
            if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                return jsonify({"status": 0, "message": "Not connected to bot"})
            return jsonify({"status": 1, "data": result['result']})
    except:
        return jsonify({"status": 0, "message": "Not connected to bot"})

# --------------------------------------- API ---------------------------------------

# -------------------------------------- Routes -------------------------------------

@blueprint.route('/')
def root():
    return redirect('/')

@blueprint.route('/index')
def index():
    return render_template('index.html')

@blueprint.route('/commands')
def commands():    
    data = app.commanddata
    prefix = app.variables.get("prefix", ["[p]"])

    return render_template("commands.html", cogs=[k['name'] for k in data], data=data, prefixes=prefix)

@blueprint.route('/credits')
def credits():
    return render_template("credits.html")

@blueprint.route('/guild/<int:guild>')
def guild(guild):
    if not session.get("id"):
        return redirect(url_for('base_blueprint.login'))
    # We won't disconnect the websocket here, even if it fails, so that the main updating thread doesnt run into issues
    try:
        request = {
            "jsonrpc": "2.0",
            "id": random.randint(1, 1000),
            "method": "DASHBOARDRPC__GET_SERVER",
            "params": [int(session['id']), int(guild)]
        }
        with app.lock:
            app.ws.send(json.dumps(request))
            result = json.loads(app.ws.recv())
            data = {}
            if 'error' in result:
                if result['error']['message'] == "Method not found":
                    data = {"status": 0, "message": "Not connected to bot"}
                else:
                    dashlog.error(result['error'])
                    data = {"status": 0, "message": "Something went wrong"}
            if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
                data = {"status": 0, "message": "Not connected to bot"}
        if not data:
            data = {"status": 1, "data": result['result']}
    except:
        data = {"status": 0, "message": "Not connected to bot"}
    return render_template("guild.html", data=data)

@blueprint.route('/<template>')
def route_template(template):

    if not session.get("id"):
        return redirect(url_for('base_blueprint.login'))

    try:

        return render_template(template + '.html')

    except TemplateNotFound:
        return render_template('page-404.html'), 404
    
    except:
        return render_template('page-500.html'), 500



# -*- encoding: utf-8 -*-
"""
License: MIT
Copyright (c) 2019 - present AppSeed.us
"""
 
import hashlib, binascii, os

# Inspiration -> https://www.vitoshacademy.com/hashing-passwords-in-python/

def hash_pass( password ):
    """Hash a password for storing."""
    salt = hashlib.sha256(os.urandom(60)).hexdigest().encode('ascii')
    pwdhash = hashlib.pbkdf2_hmac('sha512', password.encode('utf-8'), 
                                salt, 100000)
    pwdhash = binascii.hexlify(pwdhash)
    return (salt + pwdhash) # return bytes

def verify_pass(provided_password, stored_password):
    """Verify a stored password against one provided by user"""
    stored_password = stored_password.decode('ascii')
    salt = stored_password[:64]
    stored_password = stored_password[64:]
    pwdhash = hashlib.pbkdf2_hmac('sha512', 
                                  provided_password.encode('utf-8'), 
                                  salt.encode('ascii'), 
                                  100000)
    pwdhash = binascii.hexlify(pwdhash).decode('ascii')
    return pwdhash == stored_password




# -*- encoding: utf-8 -*-
"""
License: Commercial
Copyright (c) 2019 - present AppSeed.us
"""

from reddash.app import create_app
import argparse
import logging
from rich.console import Console
from rich.logging import RichHandler
import os

os.system('cls' if os.name == 'nt' else 'clear')

os.environ["WERKZEUG_RUN_MAIN"] = "true"

console = Console()

logging.basicConfig(format="%(message)s", handlers=[RichHandler()])

log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

dashlog = logging.getLogger('reddash')
dashlog.setLevel(logging.WARNING)

parser = argparse.ArgumentParser(description="Argument parser for Red Discord Bot Dashboard - Client")
parser.add_argument("--port", dest="port", type=int, default=42356)
parser.add_argument("--rpc-port", dest="rpcport", type=int, default=6133)
parser.add_argument("--interval", dest="interval", type=int, default=5, help=argparse.SUPPRESS)
parser.add_argument("--debug", dest="debug", action="store_true")
parser.add_argument("--development", dest="dev", action="store_true", help=argparse.SUPPRESS)

def main():
    args = vars(parser.parse_args())

    if args['interval'] != 5:
        console.print("Detected interval argument.  Please note that this argument should only be used if you are explicitly told to use it.", style="bold red")

    if args['dev']:
        console.print("Detected development status.  Please note that this argument is only for testing.  Do NOT use if you are opening this up to other people, as it can cause security issues.  Confirm that you wish to run the webserver in developmental status by typing 'I agree', case sensitive, below:\n", style="bold red")
        confirm = input("> ")
        print("")
        if confirm == "I agree":
            console.print("User has read the wranings and has typed I agree.  Launching developmental server...", style="bold red")
        else:
            console.print("User did not type \"I agree\".  Launching production server...", style="bold red")
            args['dev'] = False

    create_app("0.0.0.0", args['port'], args['rpcport'], args['interval'], args['debug'], args['dev'])

if __name__ == "__main__":
    main()


# -*- encoding: utf-8 -*-
"""
License: MIT
Copyright (c) 2019 - present AppSeed.us
"""

from flask import jsonify, render_template, redirect, request, url_for, session

from reddash.app.base import blueprint
from reddash.app import app
from reddash.app.base.util import verify_pass

import requests
import websocket
import json
import logging

dashlog = logging.getLogger("reddash")

@blueprint.route('/')
def route_default():
    return redirect(url_for('home_blueprint.index'))

@blueprint.route('/error-<error>')
def route_errors(error):
    return render_template('errors/{}.html'.format(error))

## Login & Registration

@blueprint.route('/callback', methods=['GET'])
def callback():
    try:
        code = request.args.get("code")
    except KeyError:
        return jsonify({"msg": "Missing code argument", "args": str(request.args)})
    redirectstr = app.variables['redirect']
    requestobj = {
        "jsonrpc": "2.0",
        "id": 0,
        "method": "DASHBOARDRPC__GET_SECRET",
        "params": []
    }
    with app.lock:
        app.ws.send(json.dumps(requestobj))
        result = json.loads(app.ws.recv())
        if 'error' in result:
            if result['error']['message'] == "Method not found":
                return jsonify({"msg": "Not connected to bot"})
            else:
                dashlog.error(result['error'])
                return jsonify({"msg": "Something went wrong"})
        if isinstance(result['result'], dict) and result['result'].get("disconnected", False):
            return jsonify({"msg": "Not connected to bot"})
        secret = result['result']['secret']
    data = {
        "client_id": int(app.variables['botid']),
        "client_secret": secret,
        "grant_type": "authorization_code",
        "code": code,
        "redirect_uri": redirectstr,
        "scope": "identify"
    }
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    response = requests.post("https://discordapp.com/api/v6/oauth2/token", data=data, headers=headers)
    try:
        token = response.json()["access_token"]
    except KeyError:
        dashlog.error(f"Failed to log someone in.\n{response.json()}")
        return jsonify({"msg": "Failed to obtain token", "returned": response.json()})
    new = requests.get("https://discordapp.com/api/v6/users/@me", headers={"Authorization": f"Bearer {token}"})
    new_data = new.json()
    if "id" in new_data:
        session['id'] = new_data['id']
        session['avatar'] = f"https://cdn.discordapp.com/avatars/{new_data['id']}/{new_data['avatar']}.png"
        session['username'] = new_data['username']
        return redirect(url_for('home_blueprint.index'))
    dashlog.error(f"Failed to obtain a user's profile.\n{new.json()}")
    return jsonify({"msg": "Failed to obtain user profile", "returned": new.json()})

@blueprint.route('/login', methods=['GET', 'POST'])
def login():
    if not session.get("id"):
        return render_template( 'login/login.html')
    return redirect(url_for('home_blueprint.index'))

@blueprint.route('/logout')
def logout():
    del session['id']
    return redirect(url_for('base_blueprint.login'))

## Errors

@blueprint.errorhandler(403)
def access_forbidden(error):
    return render_template('errors/403.html'), 403

@blueprint.errorhandler(404)
def not_found_error(error):
    return render_template('errors/404.html'), 404

@blueprint.errorhandler(500)
def internal_error(error):
    return render_template('errors/500.html'), 500



from setuptools import setup
import pathlib

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name="Red-Dashboard",
    version="0.1.3a",
    description="An easy-to-use interactive web dashboard to control your Redbot.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/NeuroAssassin/Red-Dashboard",
    author="Neuro Assassin",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8"
    ],
    packages=["reddash"],
    include_package_data=True,
    install_requires=["flask==1.1.2", "requests==2.23.0", "cryptography==2.9.2", "websocket_client==0.57.0", "waitress==1.4.3", "rich==1.3.1"],
    entry_points={
        "console_scripts": [
            "reddash=reddash.__main__:main"
        ]
    }
)